##############################################################################
#
# Copyright (c) 2012 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
import os
import tempfile
import unittest
import z3c.layer.pagelet.testing


class SystemErrorTests(unittest.TestCase):

    layer = z3c.layer.pagelet.testing.TestLayer

    def setUp(self):
        fd, self.zcml_file = tempfile.mkstemp('.zcml')
        zcml = os.fdopen(fd, 'w')
        zcml.write('''<configure />''')
        zcml.close()

    def tearDown(self):
        from zope.app.appsetup import appsetup
        os.unlink(self.zcml_file)
        appsetup.reset()

    def callVUT(self):
        from z3c.layer.pagelet.browser import SystemErrorPagelet
        from zope.publisher.browser import TestRequest
        view = SystemErrorPagelet(None, TestRequest())
        view.update()
        return view

    def test_does_not_set_HTTP_500_if_devmode_is_not_set(self):
        from zope.app.appsetup import appsetup
        appsetup.config(self.zcml_file, ())
        view = self.callVUT()
        self.assertEqual(599, view.request.response.getStatus())

    def test_sets_HTTP_500_if_config_context_is_empty(self):
        view = self.callVUT()
        self.assertEqual(500, view.request.response.getStatus())

    def test_sets_HTTP_500_if_devmode_is_set(self):
        from zope.app.appsetup import appsetup
        appsetup.config(self.zcml_file, ('devmode',))
        view = self.callVUT()
        self.assertEqual(500, view.request.response.getStatus())



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Browser Code.
"""
from z3c.layer.pagelet import interfaces
import z3c.pagelet.browser
import z3c.template.interfaces
import zope.authentication.interfaces
import zope.component
import zope.interface


def inDevMode():
    """Are we are running in debug mode? Can error messages be more telling?"""
    try:
        from zope.app.appsetup.appsetup import getConfigContext
    except ImportError:
        # We are outside a Zope 3 context, so let's play safe:
        return False

    config_context = getConfigContext()
    if config_context is None:
        # We are probably inside a test:
        return True
    return config_context.hasFeature('devmode')


@zope.interface.implementer(interfaces.ISystemErrorPagelet)
class SystemErrorPagelet(z3c.pagelet.browser.BrowserPagelet):
    """SystemError pagelet."""

    def update(self):
        if inDevMode():
            self.request.response.setStatus(500)

    def isSystemError(self):
        return True


@zope.interface.implementer(interfaces.IUnauthorizedPagelet)
class UnauthorizedPagelet(z3c.pagelet.browser.BrowserPagelet):
    """Unauthorized pagelet."""

    def update(self):
        # Set the error status to 403 (Forbidden) in the case when we don't
        # challenge the user
        self.request.response.setStatus(403)

        # make sure that squid does not keep the response in the cache
        self.request.response.setHeader(
            'Expires', 'Mon, 26 Jul 1997 05:00:00 GMT')
        self.request.response.setHeader(
            'Cache-Control', 'no-store, no-cache, must-revalidate')
        self.request.response.setHeader('Pragma', 'no-cache')

        principal = self.request.principal
        auth = zope.component.getUtility(
            zope.authentication.interfaces.IAuthentication)
        auth.unauthorized(principal.id, self.request)

    def render(self):
        if self.request.response.getStatus() not in (302, 303):
            template = zope.component.getMultiAdapter(
                (self, self.request), z3c.template.interfaces.IContentTemplate)
            return template(self)


@zope.interface.implementer(interfaces.IUserErrorPagelet)
class UserErrorPagelet(z3c.pagelet.browser.BrowserPagelet):
    """UserError pagelet."""

    def title(self):
        return self.context.__class__.__name__


@zope.interface.implementer(interfaces.INotFoundPagelet)
class NotFoundPagelet(z3c.pagelet.browser.BrowserPagelet):
    """NotFound pagelet."""

    def render(self):
        self.request.response.setStatus(404)
        template = zope.component.getMultiAdapter(
            (self, self.request), z3c.template.interfaces.IContentTemplate)
        return template(self)



##############################################################################
#
# Copyright (c) 2007-2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test Pagelet Layer
"""
import doctest
import re
import unittest
import z3c.layer.pagelet.testing
import zope.testing.renormalizing

checker = zope.testing.renormalizing.RENormalizing([
    # Python 3 renamed type to class.
    (re.compile('<type'), '<class'),
])


def create_suite(*args, **kw):
    kw['checker'] = checker
    kw['optionflags'] = (
        doctest.NORMALIZE_WHITESPACE
        | doctest.ELLIPSIS
        | doctest.IGNORE_EXCEPTION_DETAIL
    )
    kw['globs'] = dict(
        getRootFolder=z3c.layer.pagelet.testing.TestLayer.getRootFolder,
        make_wsgi_app=z3c.layer.pagelet.testing.TestLayer.make_wsgi_app)
    suite = doctest.DocFileSuite(*args, **kw)
    suite.layer = z3c.layer.pagelet.testing.TestLayer
    return suite


def test_suite():
    suite = unittest.TestSuite((
        create_suite('../README.rst'),
        create_suite('bugfixes.rst'),
    ))
    return suite



##############################################################################
#
# Copyright (c) 2007-2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test Fixures
"""

from zope.publisher.browser import BrowserPage
from zope.security.interfaces import Unauthorized
from zope.exceptions.interfaces import UserError

from z3c.pagelet import browser
import z3c.layer.pagelet


class IPageletBrowserTestSkin(z3c.layer.pagelet.IPageletBrowserLayer):
    """The pagelet layer test skin."""


class TestPage(browser.BrowserPagelet):
    """Test page."""


class UnauthorizedPage(BrowserPage):
    """Raise Unauthorized."""

    def __call__(self):
        raise Unauthorized('not authorized')


class UserErrorPage(BrowserPage):
    """Raise UserError."""

    def __call__(self):
        raise UserError('simply user error')


class SystemErrorPage(BrowserPage):
    """Raise SystemError."""

    def __call__(self):
        raise Exception('simply system error')


class ContainerContentsPage(BrowserPage):
    """Contents of a conatiner."""

    def __call__(self):
        return str([type(x) for x in self.context.values()])



##############################################################################
#
# Copyright (c) 2007-2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Setup"""
import os
from setuptools import setup, find_packages


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()


TESTS_REQUIRE = [
    'WebTest',
    'zope.app.wsgi >= 3.8',
    'zope.exceptions',
    'zope.principalregistry',
    'zope.publisher',
    'zope.security',
    'zope.securitypolicy',
    'zope.testing',
    'zope.testrunner',
]

setup(
    name='z3c.layer.pagelet',
    version='2.1',
    author='Zope Foundation and Contributors',
    author_email='zope-dev@zope.org',
    description="Pagelet layer setup for Zope 3",
    long_description=(
        read('README.rst')
        + '\n\n' +
        '.. contents::'
        + '\n\n' +
        read('src', 'z3c', 'layer', 'pagelet', 'README.rst')
        + '\n\n' +
        read('CHANGES.rst')
    ),
    keywords="z3c pagelet layer zope zope3",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Zope Public License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Topic :: Internet :: WWW/HTTP',
        'Framework :: Zope :: 3'],
    url='http://pypi.python.org/pypi/z3c.layer.pagelet',
    license='ZPL 2.1',
    packages=find_packages('src'),
    include_package_data=True,
    package_dir={'': 'src'},
    namespace_packages=['z3c', 'z3c.layer'],
    extras_require=dict(
        test=TESTS_REQUIRE,
    ),
    install_requires=[
        'setuptools',
        'z3c.pagelet',
        'z3c.template',
        'zope.authentication',
        'zope.browser>=1.2',
        'zope.browserresource',
        'zope.component',
        'zope.interface',
        'zope.login',
        'zope.publisher>=3.12',
    ],
    zip_safe=False,
)



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Pagelet Layer Interfaces
"""
import zope.interface
import zope.browser.interfaces


class IUnauthorizedPagelet(zope.interface.Interface):
    """Unauthorized pagelet marker."""


class ISystemErrorPagelet(zope.browser.interfaces.ISystemErrorView):
    """System error view pagelet marker."""


class IUserErrorPagelet(zope.interface.Interface):
    """User error pagelet marker."""


class INotFoundPagelet(zope.interface.Interface):
    """NotFound pagelet marker."""
